"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.requestMiddle = exports.getMockData = exports.createMockServer = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const chokidar_1 = __importDefault(require("chokidar"));
const chalk_1 = __importDefault(require("chalk"));
const glob_1 = __importDefault(require("glob"));
const mockjs_1 = __importDefault(require("mockjs"));
const utils_1 = require("./utils");
const loadConfigFromBundledFile_1 = require("./loadConfigFromBundledFile");
const rollup_1 = require("rollup");
const rollup_plugin_esbuild_1 = __importDefault(require("rollup-plugin-esbuild"));
const pathResolve = require('@rollup/plugin-node-resolve');
let mockData = [];
async function createMockServer(opt = { mockPath: 'mock', ignoreFiles: [], configPath: 'vite.mock.config' }) {
    opt = {
        mockPath: 'mock',
        ignoreFiles: [],
        watchFiles: true,
        supportTs: true,
        configPath: 'vite.mock.config.ts',
        ...opt,
    };
    if (mockData.length > 0)
        return;
    mockData = await getMockConfig(opt);
    const { watchFiles = true } = opt;
    if (watchFiles) {
        const watch = await createWatch(opt);
        watch && watch();
    }
}
exports.createMockServer = createMockServer;
function getMockData() {
    return mockData;
}
exports.getMockData = getMockData;
// request match
async function requestMiddle(ctx, next) {
    const path = ctx.path;
    const req = mockData.find((item) => item.url === path);
    if (req) {
        const { response, timeout } = req;
        if (timeout) {
            await utils_1.sleep(timeout);
        }
        const { body, query } = ctx.request;
        const mockRes = utils_1.isFunction(response) ? response({ body, query }) : response;
        console.log(`${chalk_1.default.green('[vite:mock-server]:request invoke: ' + ` ${chalk_1.default.cyan(path)} `)}`);
        ctx.type = 'json';
        ctx.status = 200;
        ctx.body = mockjs_1.default.mock(mockRes);
        return;
    }
    await next();
}
exports.requestMiddle = requestMiddle;
// create watch mock
function createWatch(opt) {
    const { configPath } = opt;
    const { absConfigPath, absMockPath } = getPath(opt);
    if (process.env.VITE_DISABLED_WATCH_MOCK === 'true')
        return;
    const watchDir = [];
    const exitsConfigPath = fs_1.existsSync(absConfigPath);
    exitsConfigPath && configPath ? watchDir.push(absConfigPath) : watchDir.push(absMockPath);
    const watcher = chokidar_1.default.watch(watchDir, {
        ignoreInitial: true,
    });
    const watch = () => {
        watcher.on('all', async (event, file) => {
            console.log(chalk_1.default.magentaBright(`[vite:mock-server]:mock file ${event}.`));
            mockData = await getMockConfig(opt);
        });
    };
    return watch;
}
// clear cache
function cleanRequireCache(opt) {
    const { absConfigPath, absMockPath } = getPath(opt);
    Object.keys(require.cache).forEach((file) => {
        if (file === absConfigPath || file.indexOf(absMockPath) > -1) {
            delete require.cache[file];
        }
    });
}
// load mock .ts files and watch
async function getMockConfig(opt) {
    cleanRequireCache(opt);
    const { absConfigPath, absMockPath } = getPath(opt);
    const { ignoreFiles = [], ignore, configPath, supportTs } = opt;
    let ret = [];
    if (configPath && fs_1.existsSync(absConfigPath)) {
        console.log(chalk_1.default.blue(`[vite:mock-server]:load mock data from ${absConfigPath}`));
        let resultModule = await resolveModule(absConfigPath);
        ret = resultModule;
    }
    else {
        const mockFiles = glob_1.default
            .sync(`**/*.${supportTs ? 'ts' : 'js'}`, {
            cwd: absMockPath,
            ignore: ignoreFiles,
        })
            .filter((item) => {
            if (!ignore) {
                return true;
            }
            if (utils_1.isFunction(ignore)) {
                return ignore(item);
            }
            if (utils_1.isRegExp(ignore)) {
                return !ignore.test(item);
            }
            return true;
        });
        try {
            ret = [];
            for (let index = 0; index < mockFiles.length; index++) {
                const mockFile = mockFiles[index];
                const resultModule = await resolveModule(path_1.join(absMockPath, mockFile));
                let mod = resultModule;
                if (!utils_1.isArray(mod)) {
                    mod = [mod];
                }
                ret = [...ret, ...mod];
            }
        }
        catch (error) {
            console.log(`${chalk_1.default.red('[vite:mock-server]:mock reload error!')}\n${error}`);
            ret = [];
        }
    }
    return ret;
}
// Inspired by vite
// support mock .ts files
async function resolveModule(path) {
    // use node-resolve to support .ts files
    const nodeResolve = pathResolve.nodeResolve({
        extensions: ['.mjs', '.js', '.ts', '.jsx', '.tsx', '.json'],
    });
    const bundle = await rollup_1.rollup({
        input: path,
        treeshake: false,
        plugins: [
            rollup_plugin_esbuild_1.default({
                include: /\.[jt]sx?$/,
                exclude: /node_modules/,
                sourceMap: false,
            }),
            nodeResolve,
        ],
    });
    const { output: [{ code }], } = await bundle.generate({
        exports: 'named',
        format: 'cjs',
    });
    return await loadConfigFromBundledFile_1.loadConfigFromBundledFile(path, code);
}
// get custom config file path and mock dir path
function getPath(opt) {
    const { mockPath, configPath } = opt;
    const cwd = process.cwd();
    const absMockPath = path_1.join(cwd, mockPath || '');
    const absConfigPath = path_1.join(cwd, configPath || '');
    return {
        absMockPath,
        absConfigPath,
    };
}
